#!/usr/bin/env python
# coding=utf-8

import os
import shutil


def read_file(file_path, encoding='utf-8'):
    """读取文本文件内容"""
    all_line = None
    try:
        if exists(file_path):
            fsock = open(file_path, "r", encoding=encoding)
            all_line = fsock.readlines()
            fsock.close()
    except Exception as e:
        pass
    return all_line


def read_file_line(file_path, encoding='utf-8'):
    if exists(file_path):
        with open(file_path, 'r', encoding=encoding) as f:
            while True:
                try:
                    line = f.readline()
                except Exception as e:
                    return
                else:
                    if line:
                        yield line
                    else:
                        break


def get_file_info(file_path):
    """读取文件信息"""
    try:
        if exists(file_path):
            return os.stat(file_path)
    except Exception as e:
        pass


def save_file(file_path, content, mode='a', encoding='utf-8'):
    """保存内容到文件里"""
    try:
        file_object = open(file_path, mode, encoding=encoding)
        file_object.write(content)
        file_object.close()
        return True
    except Exception as e:
        print(str(e.args))
        return False


def copy_file(file_path, save_path):
    """复制文件"""
    try:
        if shutil.copyfile(file_path, save_path):
            return True
    except Exception as e:
        pass
    return False


def move(file_path, save_path):
    """复制文件"""
    try:
        if shutil.copyfile(file_path, save_path):
            os.remove(file_path)
            return True
    except Exception as e:
        pass
    return False



def remove_file(file_path):
    """删除文件"""
    try:
        if os.path.isfile(file_path):
            os.remove(file_path)
            return True
    except:
        pass
    return False


def remove_all_file(file_list):
    """批量删除文件"""
    try:
        for file_path in file_list:
            remove_file(file_path)
    except:
        pass


def exists(file_path):
    """检查文件或文件夹是否存在"""
    return os.path.exists(file_path)


def is_file(file_path):
    """检查是否是文件"""
    try:
        return os.path.isfile(file_path)
    except:
        pass
    return False


def create_dirs(dirs_path):
    """
    创建文件夹（可一次性创建多层文件夹）
    :param dirs_path: 文件夹路径
    :return:
    """
    try:
        if not exists(dirs_path):
            os.makedirs(dirs_path)
    except:
        pass

def get_file_size(filePath):
    """
    读取文件大小
    :param filePath: 文件存储路径
    :return:
    """
    if not exists(filePath):
        return 0
    try:
        return os.path.getsize(filePath)
    except Exception as e:
        print(e.args)
        return 0


def get_dir(dir_path):
    """获取当前路径下所有子目录"""
    for root, dirs, files in os.walk(dir_path):
        return dirs


def get_dir_files(dir_path):
    """获取当前路径下所有非目录子文件"""
    for root, dirs, files in os.walk(dir_path):
        return files


def get_root_path():
    """
    获取当前项目根目录所在路径
    :return:
    """
    return os.path.abspath('.')


def get_current_path(file_path=__file__):
    """
    获取当前调用本函数的程序文件所在路径
    :param file_path: 调用本函数文件所在路径，值为：__file__
    :return:
    """
    return os.path.split(os.path.realpath(file_path))[0]