"""
机器学习K最近邻算法操作类
"""
import numpy
import operator
from collections import Counter


def knn_classify(ml_data, ml_label, test_data, k):
    """
    kNN分类算法函数
    :param ml_data: 训练数据特征集（features）
    :param ml_label: 训练数据特征标签集（labels————特征集答案）
    :param test_data: 用于knn分类测试的数据
    :param k: 选择最近邻的数目
    :return: 返回knn算法预测的结果（所对应的标签值————分类值label）
    """
    # 1. 距离计算
    data_size = ml_data.shape[0]

    # 使用numpy的tile函数，生成和训练样本对应的矩阵，并与训练样本求差
    """
    tile会将第一参数中的数组复制成指定数量的矩阵
    比如：numpy.tile(test_data, (10, 1))
        test_data = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0, 
                     0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
        当第二个参数为(10, 1)时，则表示会将创建一个行数为10的1维数组集，每一行等于test_data值的矩阵
        即：
        result = [[0,0,0,0,0,0...0,0,0,0,0,0],
                  [0,0,0,0,0,0...0,0,0,0,0,0],
                  [0,0,0,0,0,0...0,0,0,0,0,0],
                  [0,0,0,0,0,0...0,0,0,0,0,0],
                  [0,0,0,0,0,0...0,0,0,0,0,0],
                  [0,0,0,0,0,0...0,0,0,0,0,0],
                  [0,0,0,0,0,0...0,0,0,0,0,0],
                  [0,0,0,0,0,0...0,0,0,0,0,0],
                  [0,0,0,0,0,0...0,0,0,0,0,0],
                  [0,0,0,0,0,0...0,0,0,0,0,0]]

    numpy.tile(test_data, (data_size, 1))
    则会用test_data数据生成一个与ml_data一样大小的一个矩阵，用于与ml_data进行运算
    """
    data_tile = numpy.tile(test_data, (data_size, 1))

    # 将测试数据test_data生成的矩阵与训练数据特征集数据ml_data相减，求两者的不同点
    """
    由于两者都是由0和1组成的，求差时，相同位置为1的则会被减去变为0，其中一者为1另一者为0的，则会变成1或 - 1，
    这样就可以直接通过计算求出两个验证码相比较不一样的地方有哪些，如果两个验证码基本相同，则不为0的位置极少（完全一样则全部为0），
    如果两个验证码不一样，则不为0的比较多
    """
    diff_mat = data_tile - ml_data

    # 矩阵相减计算出的结果求平方值
    # 通过前面两个矩阵求差值后，得出的矩阵中的值域为-1、0和1，求平方的结果是让负数变为正数，即将全部-1转为1，方便后面统计不同点的数量
    diff_mat_square = numpy.square(diff_mat)

    # 将矩阵的每一行相加
    """
    即直接求出test_data验证码与knn学习数据集中的验证码，在转为0和1的数组中不一样的像素点的数量
    distances = [21,7,92,43,56,37,37,56,37,45,20,21,17,13,17,29,58,62,66,62,56,57,61,55,60,99,90,99,94,91,91,93,89,90,
                 ......
                 76,78,69,68,73,63,72,75,76,67,70,69,72,68,67,68,80,61,59,53,59,60,64,56,69,56,59,59] 
    """
    distances = diff_mat_square.sum(axis=1)

    # 根据距离排序从小到大的排序，返回对应的索引位置
    """
    argsort() 会将矩阵（distances）中的值从小到大排列，并提取其对应的index（索引），然后用索引值生成新的矩阵
    例如：
        y = numpy.array([8,0,52,7,66,21,36])
        矩阵y使用argsort函数进行排序后，值为其值的索引组成的矩阵
        y.argsort() = numpy.array([1,3,0,5,6,2,4])
        表示的是y[1] < y[3] < y[0] < y[5] < y[6] < y[2] < y[4]
                  0 < 7 < 8 < 21 < 36 < 52 < 66

    使用distances.argsort()即表示，通过以上的计算，根据验证码的相似度，生成了最相似的排序，排在最前面的验证码与测试验证码最相似
    distances_sort = [1,13,14,12,10,0,11,15,8,6,5,3,1009,9,986,1018,1016,987,999,1012,1000,997,1011,1010,100,1001,1007,
                      ......
                      497,19,146,17,77,984,493,112,479,305,104,1013,145,1015,59,65]
    """
    distances_sort = distances.argsort()

    # 2. 从排序中，选取距离最小的k个点（选取与测试验证码最相似的k个验证码）
    class_count = {}
    for i in range(k):
        # 从学习标签集（答案集）中提取最相似的标签值
        label = ml_label[distances_sort[i]]
        # 通过字典累加的方式，统计最相似的各个标签数量
        # class_count = {'3': 3, '6': 1, '5': 2}  表示经过相似度计算，与测试验证码相似的值中，3有3个，6有1个，5有2个
        class_count[label] = class_count.get(label, 0) + 1

    # 3. 将相似度统计结果从大到小进行排序
    # class_count_sorted = [('3', 3), ('5', 2), ('6', 1)]
    class_count_sorted = sorted(class_count.items(), key=operator.itemgetter(1), reverse=True)

    # 返回第一个（数量最多）的标签
    return class_count_sorted[0][0]


def knn_classify2(ml_data, ml_label, test_data, k):
    """
    kNN分类算法函数
    :param ml_data: 训练数据特征集（features）
    :param ml_label: 训练数据特征标签集（labels————特征集答案）
    :param test_data: 用于knn分类测试的数据
    :param k: 选择最近邻的数目
    :return: 返回knn算法预测的结果（所对应的标签值————分类值label）
    """
    # 让预测参数矩阵（test_data）对每一个训练集矩阵（ml_data）相减，并求平方值（将负数转为正数），
    # 然后对矩阵中的值执行求和运算，得出每个训练集矩阵数据与预测参数矩阵的距离值
    distances = numpy.sum((test_data - ml_data) ** 2, axis=1)
    # 将矩阵距离值（distances）从小到大排序，并提取其对应的index（索引），然后用索引值生成新的矩阵
    # 只取出排在前k位的索引值，用于ml_label提取对应的标签
    labels = [ml_label[index] for index in distances.argsort()[0: k]]
    # 使用Counter函数统计列表（labels）中，各标签出现的次数，并按从大到小排列，
    # 然后返回标签数最多的元素，将这个元素的标签返回给调用程序
    return Counter(labels).most_common(1)[0][0]


def normalization(ml_data, min_value, max_value):
    """
    对学习数据进行归一化处理
    归一化公式：
        X1 = (X-Xmin)/(Xmax-Xmin)
        其中的 min 和 max 分别是数据集中的最小特征值和最大特征值，该公司可以自动将数字特征值转化为0到1的区间
    :param ml_data: 学习数据集
    :param min_value: 学习数据集最小值
    :param max_value: 学习数据集每个属性的最大值
    :return: 归一化处理后的数据集
    """
    # 极差
    ranges = max_value - min_value

    # 获取数据集数量
    if len(ml_data.shape) == 1:
        data_total = 1
    else:
        data_total = ml_data.shape[0]
    # 生成与最小值之差组成的矩阵
    diff_mat = ml_data - numpy.tile(min_value, (data_total, 1))
    # 将最小值之差矩阵除以范围矩阵，得出归一化后的矩阵（数字特征在0到1区间的矩阵）
    distances = diff_mat / numpy.tile(ranges, (data_total, 1))
    return distances