"""
生成验证码测试图片，将图片二值化处理后，转换为32*32像素大小的文本数据集文件
"""
import numpy
from PIL import Image
import matplotlib.pylab as plt
from common import verify_helper, random_helper, file_helper


def generate_verify_code(dir_name):
    """生成验证码图片"""
    try:
        # 生成验证码图片与验证码
        verify_img, verify_code = verify_helper.create_verify_code(1, size=(32, 32), draw_lines=False, n_line=(0, 0), draw_points=False, point_chance=0)
        # 生成图片名称
        img_name = verify_code + '_' + random_helper.get_string(10) + '.jpeg'
        # 文件存储路径
        save_path = '{0}\\{1}\\'.format(file_helper.get_current_path(__file__), dir_name)
        # 检查并创建存储目录
        file_helper.create_dirs(save_path)
        # 将验证码图片存储到指定目录
        # verify_img.save(save_path + img_name, 'jpeg')
        # 将验证码图片转换成32*32大小的字符串，并存储到指定目录中
        img_to_text(verify_img, img_name, save_path)
        # 关闭图片对象
        verify_img.close()

    except Exception as e:
        print(e.args)
        pass


def img_to_text(verify_img, img_name, save_path):
    """
    将图片转化为32*32大小的字符串（用0 1表示），并存储到指定目录中
    :param verify_img: 图片对象
    :param img_name: 存储文件名称
    :param save_path: 存储文件夹路径
    """
    # 得到可编辑的图片像素集
    img = verify_img.load()

    # 对图片进行降噪，并转化为黑白两色
    for y in range(verify_img.size[1]):
        for x in range(verify_img.size[0]):
            if img[x, y][0] < 90:
                img[x, y] = (0, 0, 0, 255)
            elif img[x, y][1] < 136:
                img[x, y] = (0, 0, 0, 255)
            elif img[x, y][2] > 0:
                img[x, y] = (255, 255, 255, 255)

    # 将图片设置（放大或缩小）为32*32像素大小
    verify_img = verify_img.resize((32, 32), Image.LANCZOS)

    # 得到像素数组
    text_array = plt.array(verify_img)

    # 创建numpy数组
    gray_array = numpy.zeros((32, 32))

    # 行数
    for x in range(text_array.shape[0]):
        # 列数
        for y in range(text_array.shape[1]):
            # 按照公式将其转为0和1, 公式： 0.299 * R + 0.587 * G + 0.114 * B
            # 计算灰度，若为255则白色，数值越小越接近黑色
            gary = 0.299 * text_array[x][y][0] + 0.587 * text_array[x][y][1] + 0.114 * text_array[x][y][2]
            # 设置一个阙值，记为0
            if gary == 255:
                gray_array[x][y] = 0
            else:
                # 否则认为是黑色，记为1
                gray_array[x][y] = 1

    # 组合存储文件路径
    save_path = save_path + img_name.split('.')[0] + '.txt'
    # 保存到文件中
    numpy.savetxt(save_path, gray_array, fmt='%d', delimiter='')


if __name__ == '__main__':
    # 生成1000个验证码机器学习文件
    # 验证码学习文件需要达到一定量级后，错误率才能降到0
    for i in range(1000):
        generate_verify_code('verify_img')
    # 生成1000个验证码机器学习测试文件
    for i in range(1000):
        generate_verify_code('verify_img_test')